##Analytical script for Benson et al. (2021) "Reconstructed evolutionary patterns for crocodile-line archosaurs demonstrate impact of failure to log-transform body size data" Communications Biology. Prepared by Roger Benson, 10 August 2021 for R 4.0.3 on Mac OS X, GUI 1.73 Catalina build

require( ape )	#ape version 5.0
require( caper )	#caper version 1.0.1
	
	##Set the working directory to where the data files are kept
		setwd( "~/Documents/Stockdale & Benton response letter/Revised submission/Supplementary_Data" )
			source('edgeAges.R')	#Load required function

		tree <- read.nexus( "Crocodylian_tree.nex" )	#Load phylogeny provided by Max Stockdale, January 2021			
	
		measurements <- read.csv( "Stockdale&Benton2021pseudosuchian measurements.csv" , row.names = 1 )
			#Load measurements from Stockdale & Benton (2021)
	
 	##Load Bayestraits output and PC1 body size index for the log-transformed version of the data
		BayesTraits.output_log <- read.delim( "Crocodylian_BayesTraits_PC1_logged.txt.Output 10milgen TABLE.txt" )
			data_log <- read.delim( "Crocodylian_BayesTraits_PC1_logged.txt" , sep = " " , row.names = 1 , header = F )

			data2_log <- data_log[,1]
				names( data2_log ) <- rownames( data_log )
					data2_log <- data2_log[ tree$tip.label ]

					anc_log <- ace( data2_log , tree )	#Estimate ancestral body sizes on the tree

 	##Load Bayestraits output and PC1 body size index for the NOT log-transformed version of the data
		BayesTraits.output_nlog <- read.delim( "Crocodylian_BayesTraits_PC1_non-logged.txt.Output 10milgen TABLE.txt" )
			data_nlog <- read.delim( "Crocodylian_BayesTraits_PC1_non.logged.txt" , sep = " " , row.names = 1 , header = F  )

			data2_nlog <- data_nlog[,1]
				names( data2_nlog ) <- rownames( data_nlog )
					data2_nlog <- data2_nlog[ tree$tip.label ]

					anc_nlog <- ace( data2_nlog , tree )	#Estimate ancestral body sizes on the tree
			

	##The next part creates a function that generates the colour scales
		colourRange <- function( X , ramp = c( "black","red" ) ) {
		lowerZero <- X - min( X , na.rm = T )
		upperOne <- lowerZero / max( lowerZero )
			colour.specs <- colorRamp( ramp )( upperOne ) / 255
				for( i in 1:nrow( colour.specs ) ) {
					if( i == 1 ) { bg.out <- rgb( colour.specs[ i , 1 ] , colour.specs[ i , 2 ] , colour.specs[ i , 3 ] , 1 )
						}	else	{ bg.out[ i ] <- rgb( colour.specs[ i , 1 ] , colour.specs[ i , 2 ] , colour.specs[ i , 3 ] , 1 ) }
				}
				bg.out
			}
			
					
##You -should- be able to set the parameter using 'use.parameter' and then the script will make the plots for you. We used the mean scalar values in out figures
		
	#use.parameter <- "Median.SigV"
	#use.parameter <- "Mean.SigV"
	use.parameter <- "Mean.Scalar"
		element <- "Body size"		##This only affects the writing in the plot legends. But do be sure to change it depending on which data you read so that the plots are labelled correctly
		 ramp <- c( "blue" , "red" )	#This determines the colour range
	
		plot.data_log <- BayesTraits.output_log[ -1 , use.parameter ]
				#plot.data[ plot.data == 0 ] <- sort( plot.data )[2]
			plot.colours1_log <- colourRange( plot.data_log , ramp )
			plot.colours2_log <- colourRange( log10( plot.data_log ) , ramp )
				number.scale1_log <- rev( signif( seq( from = min( plot.data_log ) , to = max( plot.data_log ) , by = ( max( plot.data_log ) - min( plot.data_log ) ) / 4 ) , digits = 2 ) )
				colour.scale1_log <- colourRange( number.scale1_log , ramp )
				number.scale2_log <- rev( signif( seq( from = min( log10(plot.data_log) ) , to = max( log10(plot.data_log) ) , by = ( max( log10(plot.data_log) ) - min( log10(plot.data_log) ) ) / 4 ) , digits = 2 ) )
				colour.scale2_log <- colourRange( number.scale2_log , ramp )
		
		
		plot.data_nlog <- BayesTraits.output_nlog[ -1 , use.parameter ]
				#plot.data[ plot.data == 0 ] <- sort( plot.data )[2]
			plot.colours1_nlog <- colourRange( plot.data_nlog , ramp )
			plot.colours2_nlog <- colourRange( log10( plot.data_nlog ) , ramp )
				number.scale1_nlog <- rev( signif( seq( from = min( plot.data_nlog ) , to = max( plot.data_nlog ) , by = ( max( plot.data_nlog ) - min( plot.data_nlog ) ) / 4 ) , digits = 2 ) )
				colour.scale1_nlog <- colourRange( number.scale1_nlog , ramp )
				number.scale2_nlog <- rev( signif( seq( from = min( log10(plot.data_nlog) ) , to = max( log10(plot.data_nlog) ) , by = ( max( log10(plot.data_nlog) ) - min( log10(plot.data_nlog) ) ) / 4 ) , digits = 2 ) )
				colour.scale2_nlog <- colourRange( number.scale2_nlog , ramp )		
		

	##Calculate the summed root-to-tip rates for pGLS comparison of rates to body size variation
		nodepaths.temp <- nodepath( tree )
			nodepaths.temp <- lapply( nodepaths.temp , match , table = c( length(tree$tip.label) + 1 , tree$edge[,2] ) )
		
			sum.tip.rates_log <- unlist( lapply( lapply( nodepaths.temp ,
				function( BayesTraits.output , use.parameter , nodepath ) {
					BayesTraits.output[ nodepath , use.parameter ] } ,
						BayesTraits.output = BayesTraits.output_log ,
						use.parameter = use.parameter )
				, sum ) )

			sum.tip.rates_nlog <- unlist( lapply( lapply( nodepaths.temp ,
				function( BayesTraits.output , use.parameter , nodepath ) {
					BayesTraits.output[ nodepath , use.parameter ] } ,
						BayesTraits.output = BayesTraits.output_nlog ,
						use.parameter = use.parameter )
				, sum ) )
				
		data.temp <- data.frame( data2_nlog , sum.tip.rates_nlog , data2_log , sum.tip.rates_log , names = names( data2_nlog ) )
			pgls.data <- comparative.data( tree , data = data.temp , names.col = "names")

			summary( pgls( sum.tip.rates_nlog ~ data2_nlog , data = pgls.data , lambda = "ML" ) )
			summary( pgls( sum.tip.rates_log ~ data2_log , data = pgls.data , lambda = "ML" ) )


		
##Plot Figure 2, phylogenies with rates painted onto their branches
	dev.new( width = 14 , height = 9 )
		close.screen( all.screens = T )
			split.screen( c( 1 , 2 ) )
	screen( 2 )
		plot( tree , edge.col = plot.colours2_log , edge.width = 2 , no.margin = T , cex = 0.6 , show.tip.label = F )
			legend( 10 , 70 , legend = number.scale2_log , title = paste( "Body size (log10) rate", "\nlog10(" , use.parameter , ")" , sep = "") , bty = "n" , pch = 22 , pt.bg = colour.scale2_log , cex = 0.8 , pt.cex = 2.3 , col = colour.scale2_log )
				size.vals_log <- round( seq( max(data_log - min( data_log ) ) , min(data_log - min( data_log ) ) , length.out = 5 ) , 2 )
					legend( 10 , 120 , legend = size.vals_log , title = "Body size (log10)" , bty = "n" , pch = 21 , pt.bg = "yellow" , cex = 0.8 , pt.cex = size.vals_log / 7.5 )
						tiplabels( pch = 21 ,cex = ( data_log[ tree$tip.label , ] - min( data_log ) ) / 7.5 , bg = "yellow" )	
	screen( 1 )
		plot( tree , edge.col = plot.colours2_nlog , edge.width = 2 , no.margin = T , cex = 0.6  , show.tip.label = F )
			legend( 10 , 70 , legend = number.scale2_nlog , title = paste( "Body size (non-log) rate", "\nlog10(" , use.parameter , ")" , sep = "") , bty = "n" , pch = 22 , pt.bg = colour.scale2_nlog , cex = 0.8 , pt.cex = 2.3 , col = colour.scale2_nlog )
				size.vals_nlog <- round( seq( max(data_nlog - min( data_nlog ) ) , min(data_nlog - min( data_nlog ) ) , length.out = 5 ) , 2 )
					legend( 10 , 120 , legend = size.vals_nlog , title = "Body size (non-log)" , bty = "n" , pch = 21 , pt.bg = "yellow" , cex = 0.8 , pt.cex = size.vals_nlog / 150 )
						tiplabels( pch = 21 ,cex = ( data_nlog[ tree$tip.label , ] - min( data_nlog ) ) / 150 , bg = "yellow" )


##Plot more detailed phylogenies with taxon names at tips
	dev.new( width = 14 , height = 7 )
		plot( tree , edge.col = plot.colours2_nlog , edge.width = 3 , no.margin = T , cex = 0.5  , show.tip.label = T , direction = "u" , label.offset = 4)
			legend( 10 , 70 , legend = number.scale2_nlog , title = paste( "Body size (non-log) rate", "\nlog10(" , use.parameter , ")" , sep = "") , bty = "n" , pch = 22 , pt.bg = colour.scale2_nlog , cex = 0.8 , pt.cex = 2.3 , col = colour.scale2_nlog )
				size.vals_nlog <- round( seq( max(data_nlog - min( data_nlog ) ) , min(data_nlog - min( data_nlog ) ) , length.out = 5 ) , 2 )
					legend( 40 , 70 , legend = size.vals_nlog , title = "Body size (non-log)" , bty = "n" , pch = 21 , pt.bg = "yellow" , cex = 0.8 , pt.cex = size.vals_nlog / 150 )
						tiplabels( pch = 21 ,cex = ( data_nlog[ tree$tip.label , ] - min( data_nlog ) ) / 150 , bg = "yellow" )

	dev.new( width = 14 , height = 7 )
		plot( tree , edge.col = plot.colours2_log , edge.width = 3 , no.margin = T , cex = 0.5  , show.tip.label = T , direction = "u" , label.offset = 4)
			legend( 10 , 70 , legend = number.scale2_log , title = paste( "Body size (log10) rate", "\nlog10(" , use.parameter , ")" , sep = "") , bty = "n" , pch = 22 , pt.bg = colour.scale2_log , cex = 0.8 , pt.cex = 2.3 , col = colour.scale2_log )
				size.vals_nlog <- round( seq( max(data_log - min( data_log ) ) , min(data_log - min( data_log ) ) , length.out = 5 ) , 2 )
					legend( 40 , 70 , legend = size.vals_log , title = "Body size (log10)" , bty = "n" , pch = 21 , pt.bg = "yellow" , cex = 0.8 , pt.cex = size.vals_log / 7.5 )
						tiplabels( pch = 21 ,cex = ( data_log[ tree$tip.label , ] - min( data_log ) ) / 7.5 , bg = "yellow" )


##Plot Figure 1

	body.size_log <- c( data2_log , anc_log$ace )
		body.size2_log <- body.size_log[ tree$edge[,2] ]
	body.size_nlog <- c( data2_nlog , anc_nlog$ace )
		body.size2_nlog <- body.size_nlog[ tree$edge[,2] ]

		dev.new( width = 8 , height = 6 )
			close.screen( all.screens = T )
				split.screen( c( 2 , 2 ) )
		
			screen( 2 )
				par( mar = c(4,5,1,1) )
					plot( measurements[ names( data2_log ) , "Skull.length" ] , data2_log , bty = "l" , log = "x" , ylab = "PC1 size index\n(log measurements)" , xlab = "Skull length (mm)\n[logged axis]" , cex.lab = 0.85 , cex.axis = 0.8 )
						#text( measurements[ names( data2_log ) , "Skull.length" ] , data2_log , labels = names( data2_log ) , cex = 0.6 , pos = 4 )
							
			screen( 1 )
				par( mar = c(4,5,1,1) )
					plot( measurements[ names( data2_log ) , "Skull.length" ] , data2_nlog , bty = "l" , log = "x" , ylab = "PC1 size index\n(untransformed measurements)" , xlab = "Skull length (mm)\n[logged axis]" , cex.lab = 0.85 , cex.axis = 0.8 )					
										
				screen( 4 )
					par( mar = c(4,5,1,1))
						plot( body.size2_log , plot.data_log , bty = "l" , log = "y" , xlab = "PC1 size index\n(log measurements)" , ylab = "Rate\n(log-units/Ma)" , cex = 0 , ylim = range(plot.data_log) + c(0,20) , cex.lab = 0.85 , cex.axis = 0.8 )
							points( body.size2_log[-1:-length(tree$tip.label)] , plot.data_log[-1:-length(tree$tip.label)] , pch = 21 , col = "grey" )									
								points( body.size2_log[1:length(tree$tip.label)] , plot.data_log[1:length(tree$tip.label)] , pch = 21 , bg = "grey" )
									lm_log <- lm( log10( plot.data_log ) ~ body.size2_log )	
										abline( lm_log$coef , lty = 2 , col = "darkgrey" , lwd = 2 )									
											#leg_log <- paste( "rate = " , round( lm_log$coef , 3 )[2] , "body size +" , round( lm_log$coef , 2 )[1] , "\n p=0.19 (NS); R2 =" , round( summary( lm_log )$r.squared , 3 ) )
												#legend( "topleft" , legend = leg_log , bty = "n" , cex = 0.75)

					screen( 3 )
						par( mar = c(4,5,1,1))
							plot( body.size2_nlog , plot.data_nlog , bty = "l" , log = "y" , xlab = "PC1 size index\n(untransformed measurements)" , ylab = "Rate\n(units/Ma)" , cex = 0 , ylim = range( plot.data_nlog ) + c(0,1000) , cex.lab = 0.85 , cex.axis = 0.8 )
								points( body.size2_nlog[-1:-length(tree$tip.label)] , plot.data_nlog[-1:-length(tree$tip.label)] , pch = 21 , col = "grey" )			
								points( body.size2_nlog[1:length(tree$tip.label)] , plot.data_nlog[1:length(tree$tip.label)] , pch = 21 , bg = "grey" )									
									lm_nlog <- lm( log10(plot.data_nlog) ~ body.size2_nlog )									
										abline( lm_nlog$coef , lty = 2 , col = "darkgrey" , lwd = 2 )
											#leg_nlog <- paste( "rate = " , round( lm_nlog$coef , 2 )[2] , "body size +" , round( lm_nlog$coef , 2 )[1] , "\n p <0.001*; R2 =" , round( summary( lm_nlog )$r.squared , 3 ) )
											#	legend( "topleft" , legend = leg_nlog , bty = "n" , cex = 0.75)
												legend( "bottomright" , legend = c( "external edges" , "internal edges" ) , bty = "n" , cex = 0.75 , pch = 21 , pt.bg = c("grey","white") , col = c("black","grey") , pt.cex =1 )
												
												
												
												
##Test of correlation between PC1 size index (logged version) and log skull length

					skull.length.temp <- log10( as.numeric( measurements[ names( data2_log ) , "Skull.length" ] ) )
							names( skull.length.temp ) <- names( data2_log ) 
								skull.length.temp <- skull.length.temp[ !is.na( skull.length.temp ) ]
						PC1.index.temp <- data2_log[ names( skull.length.temp ) ]
							cor.test( skull.length.temp , PC1.index.temp )
